@@ -285,7 +285,6 @@ int prompt(bool verbose, CompilerOptions &cu)
             LCompilers::LocationManager lm;
             LCompilers::PassManager lpm;
             lpm.use_default_passes();
-            lpm.do_not_use_optimization_passes();
             {
                 LCompilers::LocationManager::FileLocations fl;
                 fl.in_filename = "input";
@@ -1462,8 +1461,12 @@ int compile_to_binary_fortran(const std::string &infile,
 int link_executable(const std::vector<std::string> &infiles,
     const std::string &outfile,
     const std::string &runtime_library_dir, Backend backend,
-    bool static_executable, bool link_with_gcc, bool kokkos,
-    bool verbose, CompilerOptions &compiler_options)
+    bool static_executable, bool shared_executable,
+    bool link_with_gcc, bool kokkos, bool verbose,
+    const std::vector<std::string> &lib_dirs,
+    const std::vector<std::string> &libraries,
+    const std::vector<std::string> &linker_flags,
+    CompilerOptions &compiler_options)
 {
     /*
     The `gcc` line for dynamic linking that is constructed below:
@@ -1527,12 +1530,27 @@ int link_executable(const std::vector<std::string> &infiles,
 #endif
     size_t dot_index = outfile.find_last_of(".");
     std::string file_name = outfile.substr(0, dot_index);
-    std::string extra_runtime_linker_path;
-    if (!compiler_options.runtime_linker_paths.empty()) {
-        for (auto &s: compiler_options.runtime_linker_paths) {
-            extra_runtime_linker_path += " -Wl,-rpath," + s;
+    std::string extra_linker_flags;
+    if (!linker_flags.empty()) {
+        for (auto &s: linker_flags) {
+            extra_linker_flags += " -W" + s;
         }
     }
+    std::string extra_library_flags;
+    if (!lib_dirs.empty()) {
+        for (auto &s: lib_dirs) {
+            extra_library_flags += " -L" + s;
+        }
+    }
+    if (!libraries.empty()) {
+        for (auto &s: libraries) {
+            extra_library_flags += " -l" + s;
+        }
+    }
+    if(static_executable && shared_executable) {
+        std::cout << "Cannot use static_executable and shared_executable together" << std::endl;
+        return 10;
+    }
     if (backend == Backend::llvm) {
         std::string run_cmd = "", compile_cmd = "";
         if (t == "x86_64-pc-windows-msvc") {
@@ -1568,21 +1586,27 @@ int link_executable(const std::vector<std::string> &infiles,
                 }
                 runtime_lib = "lfortran_runtime_static";
             }
+            if (shared_executable) {
+                options += " -shared ";
+            }
             compile_cmd = CC + options + " -o " + outfile + " ";
             for (auto &s : infiles) {
                 compile_cmd += s + " ";
             }
+	    if(!extra_library_flags.empty()) {
+	        compile_cmd += extra_library_flags + " ";
+	    }
             compile_cmd += + " -L"
                 + base_path + " -Wl,-rpath," + base_path;
-            if (!extra_runtime_linker_path.empty()) {
-                compile_cmd += extra_runtime_linker_path;
+            if (!extra_linker_flags.empty()) {
+                compile_cmd += extra_linker_flags;
             }
             compile_cmd += " -l" + runtime_lib + " -lm";
             run_cmd = "./" + outfile;
         }
-	if (verbose) {
+        if (verbose) {
             std::cout << compile_cmd << std::endl;
-	}
+        }
         int err = system(compile_cmd.c_str());
         if (err) {
             std::cout << "The command '" + compile_cmd + "' failed." << std::endl;
@@ -1599,10 +1623,10 @@ int link_executable(const std::vector<std::string> &infiles,
 #else
             cmd += "llvm-dwarfdump --debug-line " + file_name + ".out > ";
 #endif
-            std::string libasr_path = LCompilers::LFortran::get_runtime_library_c_header_dir() + "/../";
-            cmd += file_name + "_ldd.txt && (" + libasr_path + "dwarf_convert.py "
+            std::string dwarf_scripts_path = LCompilers::LFortran::get_dwarf_scripts_dir();
+            cmd += file_name + "_ldd.txt && (" + dwarf_scripts_path + "/dwarf_convert.py "
                 + file_name + "_ldd.txt " + file_name + "_lines.txt "
-                + file_name + "_lines.dat && " + libasr_path + "dat_convert.py "
+                + file_name + "_lines.dat && " + dwarf_scripts_path + "/dat_convert.py "
                 + file_name + "_lines.dat)";
             int status = system(cmd.c_str());
             if ( status != 0 ) {
@@ -1622,10 +1646,13 @@ int link_executable(const std::vector<std::string> &infiles,
         for (auto &s : infiles) {
             cmd += s + " ";
         }
+	if(!extra_library_flags.empty()) {
+	    cmd += extra_library_flags + " ";
+	}
         cmd += " -L" + base_path
             + " -Wl,-rpath," + base_path;
-        if (!extra_runtime_linker_path.empty()) {
-            cmd += extra_runtime_linker_path;
+        if (!extra_linker_flags.empty()) {
+            cmd += extra_linker_flags;
         }
         cmd += " -l" + runtime_lib + " -lm";
 	if (verbose) {
@@ -1642,6 +1669,9 @@ int link_executable(const std::vector<std::string> &infiles,
         if (static_executable) {
             options += " -static ";
         }
+        if (shared_executable) {
+            options += " -shared ";
+        }
         if (compiler_options.openmp) {
             options += " -fopenmp ";
         }
@@ -1654,6 +1684,9 @@ int link_executable(const std::vector<std::string> &infiles,
         for (auto &s : infiles) {
             cmd += s + " ";
         }
+	if(!extra_library_flags.empty()) {
+	    cmd += extra_library_flags + " ";
+	}
         cmd += " " + post_options + " -lm";
 	if (verbose) {
             std::cout << cmd << std::endl;
@@ -1679,6 +1712,9 @@ int link_executable(const std::vector<std::string> &infiles,
         for (auto &s : infiles) {
             cmd += s + " ";
         }
+	if(!extra_library_flags.empty()) {
+	    cmd += extra_library_flags + " ";
+	}
         cmd += " -L" + base_path
             + " -Wl,-rpath," + base_path;
         cmd += " -l" + runtime_lib + " -lm";
@@ -1859,50 +1895,10 @@ EMSCRIPTEN_KEEPALIVE char* emit_wasm_from_source(char *input) {
 
 #endif
 
-void _parse_linker_args(std::string& arg, std::vector<std::string>& linker_flags) {
-    if (arg == "") return;
-    std::string current_arg = "";
-    for( size_t i = 0; i < arg.size(); i++ ) {
-        char ch = arg[i];
-        if (ch != ',') {
-            current_arg.push_back(ch);
-        } else {
-            linker_flags.push_back(current_arg);
-            current_arg.clear();
-        }
-    }
-    if (!current_arg.empty()) {
-        linker_flags.push_back(current_arg);
-    }
-}
-
-void _get_rpath_from_linker_flags(std::vector<std::string>& linker_flags, CompilerOptions &co) {
-    if (!linker_flags.empty() && linker_flags[0] != "l") {
-        LCompilers::LCompilersException("Linker arguments must start with -Wl");
-        return;
-    }
-    if (linker_flags.empty()) {
-        return;
-    }
-    bool is_rpath = false;
-    for( size_t i = 1; i < linker_flags.size(); i++ ) {
-        if (linker_flags[i][0] == '-') {
-            if (linker_flags[i] == "-rpath") {
-                is_rpath = true;
-            } else {
-                is_rpath = false;
-            }
-            continue;
-        }
-        if (is_rpath) {
-            co.runtime_linker_paths.push_back(linker_flags[i]);
-        }
-    }
-}
-
 int main_app(int argc, char *argv[]) {
     int dirname_length;
     LCompilers::LFortran::get_executable_path(LCompilers::binary_executable_path, dirname_length);
+    LCompilers::LFortran::set_exec_path_and_mode(LCompilers::binary_executable_path, dirname_length);
 
     // TODO: This is now in compiler options and can be removed
     std::string runtime_library_dir = LCompilers::LFortran::get_runtime_library_dir();
@@ -1936,12 +1932,16 @@ int main_app(int argc, char *argv[]) {
     bool show_fortran = false;
     bool time_report = false;
     bool static_link = false;
+    bool shared_link = false;
     std::string skip_pass;
     std::string arg_backend = "llvm";
     std::string arg_kernel_f;
     bool print_targets = false;
     bool link_with_gcc = false;
     bool fixed_form_infer = false;
+    bool cpp = false;
+    bool cpp_infer = false;
+    bool no_cpp = false;
 
     std::string arg_fmt_file;
     int arg_fmt_indent = 4;
@@ -1957,6 +1957,7 @@ int main_app(int argc, char *argv[]) {
     std::string arg_pywrap_array_order="f";
     std::vector<std::string> linker_flags;
     std::vector<std::string> f_flags;
+    std::vector<std::string> O_flags;
 
     CompilerOptions compiler_options;
     compiler_options.po.runtime_library_dir = LCompilers::LFortran::get_runtime_library_dir();
@@ -1973,8 +1974,8 @@ int main_app(int argc, char *argv[]) {
     app.add_option("-o", compiler_options.arg_o, "Specify the file to place the output into");
     app.add_flag("-v", arg_v, "Be more verbose");
     app.add_flag("-E", arg_E, "Preprocess only; do not compile, assemble or link");
-    app.add_option("-l", arg_l, "Link library option");
-    app.add_option("-L", arg_L, "Library path option");
+    app.add_option("-l", arg_l, "Link library option")->allow_extra_args(false);
+    app.add_option("-L", arg_L, "Library path option")->allow_extra_args(false);
     app.add_option("-I", compiler_options.po.include_dirs, "Include path")->allow_extra_args(false);
     app.add_option("-J", compiler_options.po.mod_files_dir, "Where to save mod files");
     app.add_flag("-g", compiler_options.emit_debug_info, "Compile with debugging information");
@@ -1984,9 +1985,12 @@ int main_app(int argc, char *argv[]) {
     app.add_flag("--version", arg_version, "Display compiler version information");
     app.add_option("-W", linker_flags, "Linker flags")->allow_extra_args(false);
     app.add_option("-f", f_flags, "All `-f*` flags (only -fPIC supported for now)")->allow_extra_args(false);
+    app.add_option("-O", O_flags, "Optimization level (ignored for now)")->allow_extra_args(false);
 
     // LFortran specific options
-    app.add_flag("--cpp", compiler_options.c_preprocessor, "Enable C preprocessing");
+    app.add_flag("--cpp", cpp, "Enable C preprocessing");
+    app.add_flag("--cpp-infer", cpp_infer, "Use heuristics to infer if a file needs preprocessing");
+    app.add_flag("--no-cpp", no_cpp, "Disable C preprocessing");
     app.add_flag("--fixed-form", compiler_options.fixed_form, "Use fixed form Fortran source parsing");
     app.add_flag("--fixed-form-infer", fixed_form_infer, "Use heuristics to infer if a file is in fixed form");
     app.add_flag("--no-prescan", arg_no_prescan, "Turn off prescan");
@@ -2015,6 +2019,7 @@ int main_app(int argc, char *argv[]) {
     app.add_flag("--symtab-only", compiler_options.symtab_only, "Only create symbol tables in ASR (skip executable stmt)");
     app.add_flag("--time-report", time_report, "Show compilation time report");
     app.add_flag("--static", static_link, "Create a static executable");
+    app.add_flag("--shared", shared_link, "Create a shared executable");
     app.add_flag("--no-warnings", compiler_options.no_warnings, "Turn off all warnings");
     app.add_flag("--no-error-banner", compiler_options.no_error_banner, "Turn off error banner");
     app.add_option("--error-format", compiler_options.error_format, "Control how errors are produced (human, short)")->capture_default_str();
@@ -2081,12 +2086,6 @@ int main_app(int argc, char *argv[]) {
     app.require_subcommand(0, 1);
     CLI11_PARSE(app, argc, argv);
     lcompilers_unique_ID = compiler_options.generate_object_code ? get_unique_ID() : "";
-    if (!linker_flags.empty()) {
-        std::string linker_flags_args = linker_flags[0];
-        linker_flags.clear();
-        _parse_linker_args(linker_flags_args, linker_flags);
-        _get_rpath_from_linker_flags(linker_flags, compiler_options);
-    }
 
     if (arg_version) {
         std::string version = LFORTRAN_VERSION;
@@ -2116,14 +2115,17 @@ int main_app(int argc, char *argv[]) {
         if (f_flag == "PIC") {
             // Position Independent Code
             // We do this by default, so we ignore for now
+        } else if (f_flag == "default-integer-8") {
+            compiler_options.po.default_integer_kind = 8;
         } else {
             std::cerr << "The flag `-f" << f_flag << "` is not supported" << std::endl;
             return 1;
         }
     }
 
-    if( compiler_options.po.fast ) {
-        lfortran_pass_manager.use_optimization_passes();
+    if(static_link && shared_link) {
+        std::cerr << "Options '--static' and '--shared' cannot be used together" << std::endl;
+        return 1;
     }
 
     if (fmt) {
@@ -2197,6 +2199,20 @@ int main_app(int argc, char *argv[]) {
         compiler_options.fixed_form = true;
     }
 
+    if (cpp && no_cpp) {
+        throw LCompilers::LCompilersException("Cannot use --cpp and --no-cpp at the same time");
+    } else if(cpp) {
+        compiler_options.c_preprocessor = true;
+    } else if(no_cpp) {
+        compiler_options.c_preprocessor = false;
+    // Decide if a file gets preprocessing based on the extension
+    // Gfortran does the same thing
+    } else if (cpp_infer && (endswith(arg_file, ".F90") || endswith(arg_file, ".F"))) {
+        compiler_options.c_preprocessor = true;
+    } else {
+        compiler_options.c_preprocessor = false;
+    }
+
     std::string outfile;
     std::filesystem::path basename = std::filesystem::path(arg_file).filename();
     if (compiler_options.arg_o.size() > 0) {
@@ -2322,7 +2338,7 @@ int main_app(int argc, char *argv[]) {
         }
     }
 
-    if (endswith(arg_file, ".f90") || endswith(arg_file, ".f")) {
+    if (endswith(arg_file, ".f90") || endswith(arg_file, ".f") || endswith(arg_file, ".F90") || endswith(arg_file, ".F")) {
         if (backend == Backend::x86) {
             return compile_to_binary_x86(arg_file, outfile,
                     time_report, compiler_options);
@@ -2353,10 +2369,12 @@ int main_app(int argc, char *argv[]) {
         }
         if (err) return err;
         return link_executable({tmp_o}, outfile, runtime_library_dir,
-                backend, static_link, link_with_gcc, true, arg_v, compiler_options);
+                backend, static_link, shared_link, link_with_gcc, true, arg_v, arg_L,
+		arg_l, linker_flags, compiler_options);
     } else {
         return link_executable(arg_files, outfile, runtime_library_dir,
-                backend, static_link, link_with_gcc, true, arg_v, compiler_options);
+                backend, static_link, shared_link, link_with_gcc, true, arg_v, arg_L,
+		arg_l, linker_flags, compiler_options);
     }
     return 0;
 }
