@@ -51,9 +51,7 @@ Result<ASR::TranslationUnit_t*> body_visitor(Allocator &al,
 
 void load_rtlib(Allocator &al, ASR::TranslationUnit_t &tu, CompilerOptions &compiler_options) {
     SymbolTable *tu_symtab = tu.m_symtab;
-    const std::string m_kind = "lfortran_intrinsic_kind";
     const std::string m_builtin = "lfortran_intrinsic_builtin";
-    const std::string m_trig = "lfortran_intrinsic_trig";
     const std::string m_math = "lfortran_intrinsic_math";
     const std::string m_math2 = "lfortran_intrinsic_math2";
     const std::string m_math3 = "lfortran_intrinsic_math3";
@@ -61,7 +59,7 @@ void load_rtlib(Allocator &al, ASR::TranslationUnit_t &tu, CompilerOptions &comp
     const std::string m_bit = "lfortran_intrinsic_bit";
     const std::string m_ieee_arithmetic = "lfortran_intrinsic_ieee_arithmetic";
     std::vector<std::string> intrinsic_modules = {
-        m_trig, m_math2,
+        m_math2,
     };
     for (auto &module_name : intrinsic_modules) {
         Location loc;
@@ -112,7 +110,7 @@ Result<ASR::TranslationUnit_t*> ast_to_asr(Allocator &al,
             LCOMPILERS_ASSERT(diagnostics.has_error());
             throw LCompilersException("Fortran code could not be generated after symbol_table_visitor");
         }
-        std::ofstream outfile ("pass_00_initial_code_01.f90");
+        std::ofstream outfile ("pass_fortran_00_initial_code_01.f90");
         outfile << "! Fortran code after SymbolTable Visitor\n" << fortran_code.result << "\n";
         outfile.close();
     }
@@ -142,7 +140,7 @@ Result<ASR::TranslationUnit_t*> ast_to_asr(Allocator &al,
                 LCOMPILERS_ASSERT(diagnostics.has_error());
                 throw LCompilersException("Fortran code could not be generated after body_visitor");
             }
-            std::ofstream outfile ("pass_00_initial_code_02.f90");
+            std::ofstream outfile ("pass_fortran_00_initial_code_02.f90");
             outfile << "! Fortran code after Body Visitor\n" << fortran_code.result << "\n";
             outfile.close();
         }
