@@ -51,6 +51,8 @@ private:
     std::set<std::pair<uint64_t, std::string>> const_assigned;
 
     bool symbol_visited;
+    bool _return_var_or_intent_out = false;
+    bool _processing_dims = false;
 
 public:
     VerifyVisitor(bool check_external, diag::Diagnostics &diagnostics) : check_external{check_external},
@@ -426,6 +428,7 @@ public:
             LCOMPILERS_ASSERT(a.second);
             this->visit_symbol(*a.second);
         }
+        visit_ttype(*x.m_function_signature);
         for (size_t i=0; i<x.n_args; i++) {
             LCOMPILERS_ASSERT(x.m_args[i]);
             visit_expr(*x.m_args[i]);
@@ -441,15 +444,15 @@ public:
         verify_unique_dependencies(x.m_dependencies, x.n_dependencies,
                                    x.m_name, x.base.base.loc);
 
-        // Get the x symtab.
-        SymbolTable *x_symtab = x.m_symtab;
+        // Get the x parent symtab.
+        SymbolTable *x_parent_symtab = x.m_symtab->parent;
 
         // Dependencies of the function should be from function's parent symbol table.
         for( size_t i = 0; i < x.n_dependencies; i++ ) {
             std::string found_dep = x.m_dependencies[i];
 
             // Get the symbol of the found_dep.
-            ASR::symbol_t* dep_sym = x_symtab->resolve_symbol(found_dep);
+            ASR::symbol_t* dep_sym = x_parent_symtab->resolve_symbol(found_dep);
 
             require(dep_sym != nullptr,
                             "Dependency " + found_dep +  " is inside symbol table " + std::string(x.m_name));
@@ -656,7 +659,11 @@ public:
             visit_expr(*x.m_symbolic_value);
         if (x.m_value)
              visit_expr(*x.m_value);
+        _return_var_or_intent_out = x.m_intent == ASR::intentType::Out ||
+                                    x.m_intent == ASR::intentType::InOut ||
+                                    x.m_intent == ASR::intentType::ReturnVar;
         visit_ttype(*x.m_type);
+        _return_var_or_intent_out = false;
 
         verify_unique_dependencies(x.m_dependencies, x.n_dependencies,
                                    x.m_name, x.base.base.loc);
@@ -889,11 +896,17 @@ public:
         }
 
         SymbolTable* temp_scope = current_symtab;
-        
-        if (temp_scope->get_counter() != ASRUtils::symbol_parent_symtab(x.m_name)->get_counter() &&
-            !ASR::is_a<ASR::AssociateBlock_t>(*asr_owner_sym) && !ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name) &&
-                !ASR::is_a<ASR::Variable_t>(*x.m_name)) {
-            function_dependencies.push_back(std::string(ASRUtils::symbol_name(x.m_name)));
+
+        if (asr_owner_sym && temp_scope->get_counter() != ASRUtils::symbol_parent_symtab(x.m_name)->get_counter() &&
+            !ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name) && !ASR::is_a<ASR::Variable_t>(*x.m_name)) {
+            if (ASR::is_a<ASR::AssociateBlock_t>(*asr_owner_sym) || ASR::is_a<ASR::Block_t>(*asr_owner_sym)) {
+                temp_scope = temp_scope->parent;
+                if (temp_scope->get_counter() != ASRUtils::symbol_parent_symtab(x.m_name)->get_counter()) {
+                    function_dependencies.push_back(std::string(ASRUtils::symbol_name(x.m_name)));
+                }
+            } else {
+                function_dependencies.push_back(std::string(ASRUtils::symbol_name(x.m_name)));
+            }
         }
 
         if( ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name) ) {
@@ -906,6 +919,12 @@ public:
         verify_args(x);
     }
 
+    void visit_AssociateBlockCall(const AssociateBlockCall_t &x) {
+        require(symtab_in_scope(current_symtab, x.m_m),
+            "AssociateBlockCall::m_name '" + std::string(symbol_name(x.m_m)) +
+                "' cannot point outside of its symbol table");
+    }
+
     SymbolTable *get_dt_symtab(ASR::symbol_t *dt) {
         LCOMPILERS_ASSERT(dt)
         SymbolTable *symtab = ASRUtils::symbol_symtab(ASRUtils::symbol_get_past_external(dt));
@@ -1001,16 +1020,16 @@ public:
         }
     }
 
-    void visit_IntrinsicScalarFunction(const ASR::IntrinsicScalarFunction_t& x) {
+    void visit_IntrinsicElementalFunction(const ASR::IntrinsicElementalFunction_t& x) {
         if( !check_external ) {
-            BaseWalkVisitor<VerifyVisitor>::visit_IntrinsicScalarFunction(x);
+            BaseWalkVisitor<VerifyVisitor>::visit_IntrinsicElementalFunction(x);
             return ;
         }
-        ASRUtils::verify_function verify_ = ASRUtils::IntrinsicScalarFunctionRegistry
+        ASRUtils::verify_function verify_ = ASRUtils::IntrinsicElementalFunctionRegistry
             ::get_verify_function(x.m_intrinsic_id);
         LCOMPILERS_ASSERT(verify_ != nullptr);
         verify_(x, diagnostics);
-        BaseWalkVisitor<VerifyVisitor>::visit_IntrinsicScalarFunction(x);
+        BaseWalkVisitor<VerifyVisitor>::visit_IntrinsicElementalFunction(x);
     }
 
     void visit_IntrinsicArrayFunction(const ASR::IntrinsicArrayFunction_t& x) {
@@ -1034,10 +1053,21 @@ public:
         }
 
         SymbolTable* temp_scope = current_symtab;
-        
+
         if (asr_owner_sym && temp_scope->get_counter() != ASRUtils::symbol_parent_symtab(x.m_name)->get_counter() &&
-            !ASR::is_a<ASR::AssociateBlock_t>(*asr_owner_sym) && !ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name) &&
-                !ASR::is_a<ASR::Variable_t>(*x.m_name)) {
+            !ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name) && !ASR::is_a<ASR::Variable_t>(*x.m_name)) {
+            if (ASR::is_a<ASR::AssociateBlock_t>(*asr_owner_sym) || ASR::is_a<ASR::Block_t>(*asr_owner_sym)) {
+                temp_scope = temp_scope->parent;
+                if (temp_scope->get_counter() != ASRUtils::symbol_parent_symtab(x.m_name)->get_counter()) {
+                    function_dependencies.push_back(std::string(ASRUtils::symbol_name(x.m_name)));
+                }
+            } else {
+                function_dependencies.push_back(std::string(ASRUtils::symbol_name(x.m_name)));
+            }
+        }
+        if (_return_var_or_intent_out  && _processing_dims &&
+            temp_scope->get_counter() != ASRUtils::symbol_parent_symtab(x.m_name)->get_counter() &&
+            !ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name)) {
             function_dependencies.push_back(std::string(ASRUtils::symbol_name(x.m_name)));
         }
 
@@ -1085,10 +1115,30 @@ public:
             symbol_owner);
     }
 
+    void visit_ArrayConstructor(const ArrayConstructor_t& x) {
+        require(ASRUtils::is_array(x.m_type),
+            "Type of ArrayConstructor must be an array");
+        BaseWalkVisitor<VerifyVisitor>::visit_ArrayConstructor(x);
+    }
+
     void visit_ArrayConstant(const ArrayConstant_t& x) {
         require(ASRUtils::is_array(x.m_type),
             "Type of ArrayConstant must be an array");
-        BaseWalkVisitor<VerifyVisitor>::visit_ArrayConstant(x);
+
+        for (size_t i = 0; i < x.n_args; i++) {
+            require(!ASR::is_a<ASR::ArrayConstant_t>(*x.m_args[i]),
+                "ArrayConstant cannot have ArrayConstant as its elements");
+            require(
+                ASR::is_a<ASR::IntegerConstant_t>(*x.m_args[i]) ||
+                ASR::is_a<ASR::RealConstant_t>(*x.m_args[i]) ||
+                ASR::is_a<ASR::ComplexConstant_t>(*x.m_args[i]) ||
+                ASR::is_a<ASR::LogicalConstant_t>(*x.m_args[i]) ||
+                ASR::is_a<ASR::StringConstant_t>(*x.m_args[i]) ||
+                ASR::is_a<ASR::UnsignedIntegerConstant_t>(*x.m_args[i]),
+                "ArrayConstant must have constant values");
+        }
+
+        visit_ttype(*x.m_type);
     }
 
     void visit_dimension(const dimension_t &x) {
@@ -1113,9 +1163,11 @@ public:
         visit_ttype(*x.m_type);
         require(x.n_dims != 0, "Array type cannot have 0 dimensions.")
         require(!ASR::is_a<ASR::Array_t>(*x.m_type), "Array type cannot be nested.")
+        _processing_dims = true;
         for (size_t i = 0; i < x.n_dims; i++) {
             visit_dimension(x.m_dims[i]);
         }
+        _processing_dims = false;
     }
 
     void visit_Pointer(const Pointer_t &x) {
