@@ -28,8 +28,9 @@
 #include <libasr/pass/replace_arr_slice.h>
 #include <libasr/pass/replace_flip_sign.h>
 #include <libasr/pass/replace_div_to_mul.h>
-#include <libasr/pass/replace_symbolic.h>
+// #include <libasr/pass/replace_symbolic.h>
 #include <libasr/pass/replace_intrinsic_function.h>
+#include <libasr/pass/replace_intrinsic_subroutine.h>
 #include <libasr/pass/replace_fma.h>
 #include <libasr/pass/loop_unroll.h>
 #include <libasr/pass/replace_sign_from_value.h>
@@ -50,6 +51,8 @@
 #include <libasr/pass/unique_symbols.h>
 #include <libasr/pass/insert_deallocate.h>
 #include <libasr/pass/replace_print_struct_type.h>
+#include <libasr/pass/promote_allocatable_to_nonallocatable.h>
+#include <libasr/pass/replace_function_call_in_declaration.h>
 #include <libasr/codegen/asr_to_fortran.h>
 #include <libasr/asr_verify.h>
 #include <libasr/pickle.h>
@@ -76,8 +79,9 @@ namespace LCompilers {
             {"global_stmts", &pass_wrap_global_stmts},
             {"implied_do_loops", &pass_replace_implied_do_loops},
             {"array_op", &pass_replace_array_op},
-            {"symbolic", &pass_replace_symbolic},
+            // {"symbolic", &pass_replace_symbolic},
             {"intrinsic_function", &pass_replace_intrinsic_function},
+            {"intrinsic_subroutine", &pass_replace_intrinsic_subroutine},
             {"arr_slice", &pass_replace_arr_slice},
             {"print_arr", &pass_replace_print_arr},
             {"print_list_tuple", &pass_replace_print_list_tuple},
@@ -101,12 +105,13 @@ namespace LCompilers {
             {"init_expr", &pass_replace_init_expr},
             {"nested_vars", &pass_nested_vars},
             {"where", &pass_replace_where},
+            {"function_call_in_declaration", &pass_replace_function_call_in_declaration},
             {"print_struct_type", &pass_replace_print_struct_type},
             {"unique_symbols", &pass_unique_symbols},
-            {"insert_deallocate", &pass_insert_deallocate}
+            {"insert_deallocate", &pass_insert_deallocate},
+            {"promote_allocatable_to_nonallocatable", &pass_promote_allocatable_to_nonallocatable}
         };
 
-        bool is_fast;
         bool apply_default_passes;
         bool c_skip_pass; // This will contain the passes that are to be skipped in C
 
@@ -115,7 +120,7 @@ namespace LCompilers {
 
         void apply_passes(Allocator& al, ASR::TranslationUnit_t* asr,
                            std::vector<std::string>& passes, PassOptions &pass_options,
-                           [[maybe_unused]] diag::Diagnostics &diagnostics, LocationManager &lm) {
+                           [[maybe_unused]] diag::Diagnostics &diagnostics) {
             if (pass_options.pass_cumulative) {
                 int _pass_max_idx = -1, _opt_max_idx = -1;
                 for (std::string &current_pass: passes) {
@@ -158,44 +163,6 @@ namespace LCompilers {
                     std::cerr << "ASR Pass starts: '" << passes[i] << "'\n";
                 }
                 _passes_db[passes[i]](al, *asr, pass_options);
-                if (pass_options.dump_all_passes) {
-                    std::string str_i = std::to_string(i+1);
-                    if ( i < 9 )  str_i = "0" + str_i;
-                    if (pass_options.json) {
-                        std::ofstream outfile ("pass_json_" + str_i + "_" + passes[i] + ".json");
-                        outfile << pickle_json(*asr, lm, pass_options.no_loc, pass_options.with_intrinsic_mods) << "\n";
-                        outfile.close();
-                    }
-                    if (pass_options.tree) {
-                        std::ofstream outfile ("pass_tree_" + str_i + "_" + passes[i] + ".txt");
-                        outfile << pickle_tree(*asr, false, pass_options.with_intrinsic_mods) << "\n";
-                        outfile.close();
-                    }
-                    if (pass_options.visualize) {
-                        std::string json = pickle_json(*asr, lm, pass_options.no_loc, pass_options.with_intrinsic_mods);
-                        std::ofstream outfile ("pass_viz_" + str_i + "_" + passes[i] + ".html");
-                        outfile << generate_visualize_html(json) << "\n";
-                        outfile.close();
-                    }
-                    std::ofstream outfile ("pass_" + str_i + "_" + passes[i] + ".clj");
-                    outfile << ";; ASR after applying the pass: " << passes[i]
-                        << "\n" << pickle(*asr, false, true, pass_options.with_intrinsic_mods) << "\n";
-                    outfile.close();
-                }
-                if (pass_options.dump_fortran) {
-                    LCompilers::Result<std::string> fortran_code = asr_to_fortran(*asr, diagnostics, false, 4);
-                    if (!fortran_code.ok) {
-                        LCOMPILERS_ASSERT(diagnostics.has_error());
-                        throw LCompilersException("Fortran code could not be generated after pass: "
-                        + passes[i]);
-                    }
-                    std::string str_i = std::to_string(i+1);
-                    if ( i < 9 )  str_i = "0" + str_i;
-                    std::ofstream outfile ("pass_" + str_i + "_" + passes[i] + ".f90");
-                    outfile << "! Fortran code after applying the pass: " << passes[i]
-                        << "\n" << fortran_code.result << "\n";
-                    outfile.close();
-                }
 #if defined(WITH_LFORTRAN_ASSERT)
                 if (!asr_verify(*asr, true, diagnostics)) {
                     std::cerr << diagnostics.render2();
@@ -234,20 +201,23 @@ namespace LCompilers {
             }
         }
 
-        PassManager(): is_fast{false}, apply_default_passes{false},
+        PassManager(): apply_default_passes{false},
             c_skip_pass{false} {
             _passes = {
                 "nested_vars",
                 "global_stmts",
+                "transform_optional_argument_functions",
                 "init_expr",
                 "implied_do_loops",
                 "class_constructor",
                 "pass_list_expr",
                 "where",
+                "function_call_in_declaration",
                 "subroutine_from_function",
                 "array_op",
-                "symbolic",
+                // "symbolic",
                 "intrinsic_function",
+                "intrinsic_subroutine",
                 "subroutine_from_function",
                 "array_op",
                 "pass_array_by_data",
@@ -261,23 +231,25 @@ namespace LCompilers {
                 "select_case",
                 "inline_function_calls",
                 "unused_functions",
-                "transform_optional_argument_functions",
                 "unique_symbols",
-                "insert_deallocate"
+                "insert_deallocate",
             };
 
             _with_optimization_passes = {
                 "nested_vars",
                 "global_stmts",
+                "transform_optional_argument_functions",
                 "init_expr",
                 "implied_do_loops",
                 "class_constructor",
                 "pass_list_expr",
                 "where",
+                "function_call_in_declaration",
                 "subroutine_from_function",
                 "array_op",
-                "symbolic",
+                // "symbolic",
                 "intrinsic_function",
+                "intrinsic_subroutine",
                 "subroutine_from_function",
                 "array_op",
                 "pass_array_by_data",
@@ -296,10 +268,10 @@ namespace LCompilers {
                 "sign_from_value",
                 "div_to_mul",
                 "fma",
-                "transform_optional_argument_functions",
                 "inline_function_calls",
                 "unique_symbols",
-                "insert_deallocate"
+                "insert_deallocate",
+                "promote_allocatable_to_nonallocatable"
             };
 
             // These are re-write passes which are already handled
@@ -323,28 +295,88 @@ namespace LCompilers {
 
         void apply_passes(Allocator& al, ASR::TranslationUnit_t* asr,
                           PassOptions& pass_options,
-                          diag::Diagnostics &diagnostics, LocationManager &lm) {
+                          diag::Diagnostics &diagnostics) {
             if( !_user_defined_passes.empty() ) {
-                pass_options.fast = true;
                 apply_passes(al, asr, _user_defined_passes, pass_options,
-                    diagnostics, lm);
+                    diagnostics);
             } else if( apply_default_passes ) {
-                pass_options.fast = is_fast;
-                if( is_fast ) {
+                if( pass_options.fast ) {
                     apply_passes(al, asr, _with_optimization_passes, pass_options,
-                        diagnostics, lm);
+                        diagnostics);
                 } else {
-                    apply_passes(al, asr, _passes, pass_options, diagnostics, lm);
+                    apply_passes(al, asr, _passes, pass_options, diagnostics);
                 }
             }
         }
 
-        void use_optimization_passes() {
-            is_fast = true;
-        }
-
-        void do_not_use_optimization_passes() {
-            is_fast = false;
+        void dump_all_passes(Allocator& al, ASR::TranslationUnit_t* asr,
+                           PassOptions &pass_options,
+                           [[maybe_unused]] diag::Diagnostics &diagnostics, LocationManager &lm) {
+            std::vector<std::string> passes;
+            if (pass_options.fast) {
+                passes = _with_optimization_passes;
+            } else {
+                passes = _passes;
+            }
+            for (size_t i = 0; i < passes.size(); i++) {
+                // TODO: rework the whole pass manager: construct the passes
+                // ahead of time (not at the last minute), and remove this much
+                // earlier
+                // Note: this is not enough for rtlib, we also need to include
+                // it
+                if (pass_options.verbose) {
+                    std::cerr << "ASR Pass starts: '" << passes[i] << "'\n";
+                }
+                _passes_db[passes[i]](al, *asr, pass_options);
+                if (pass_options.dump_all_passes) {
+                    std::string str_i = std::to_string(i+1);
+                    if ( i < 9 )  str_i = "0" + str_i;
+                    if (pass_options.json) {
+                        std::ofstream outfile ("pass_json_" + str_i + "_" + passes[i] + ".json");
+                        outfile << pickle_json(*asr, lm, pass_options.no_loc, pass_options.with_intrinsic_mods) << "\n";
+                        outfile.close();
+                    }
+                    if (pass_options.tree) {
+                        std::ofstream outfile ("pass_tree_" + str_i + "_" + passes[i] + ".txt");
+                        outfile << pickle_tree(*asr, false, pass_options.with_intrinsic_mods) << "\n";
+                        outfile.close();
+                    }
+                    if (pass_options.visualize) {
+                        std::string json = pickle_json(*asr, lm, pass_options.no_loc, pass_options.with_intrinsic_mods);
+                        std::ofstream outfile ("pass_viz_" + str_i + "_" + passes[i] + ".html");
+                        outfile << generate_visualize_html(json) << "\n";
+                        outfile.close();
+                    }
+                    std::ofstream outfile ("pass_" + str_i + "_" + passes[i] + ".clj");
+                    outfile << ";; ASR after applying the pass: " << passes[i]
+                        << "\n" << pickle(*asr, false, true, pass_options.with_intrinsic_mods) << "\n";
+                    outfile.close();
+                }
+                if (pass_options.dump_fortran) {
+                    LCompilers::Result<std::string> fortran_code = asr_to_fortran(*asr, diagnostics, false, 4);
+                    if (!fortran_code.ok) {
+                        LCOMPILERS_ASSERT(diagnostics.has_error());
+                        throw LCompilersException("Fortran code could not be generated after pass: "
+                        + passes[i]);
+                    }
+                    std::string str_i = std::to_string(i+1);
+                    if ( i < 9 )  str_i = "0" + str_i;
+                    std::ofstream outfile ("pass_fortran_" + str_i + "_" + passes[i] + ".f90");
+                    outfile << "! Fortran code after applying the pass: " << passes[i]
+                        << "\n" << fortran_code.result << "\n";
+                    outfile.close();
+                }
+#if defined(WITH_LFORTRAN_ASSERT)
+                if (!asr_verify(*asr, true, diagnostics)) {
+                    std::cerr << diagnostics.render2();
+                    throw LCompilersException("Verify failed in the pass: "
+                        + passes[i]);
+                };
+#endif
+                if (pass_options.verbose) {
+                    std::cerr << "ASR Pass ends: '" << passes[i] << "'\n";
+                }
+            }
         }
 
         void use_default_passes(bool _c_skip_pass=false) {
@@ -360,6 +392,9 @@ namespace LCompilers {
             apply_default_passes = false;
         }
 
+        void use_fortran_passes() {
+            _user_defined_passes.push_back("unique_symbols");
+        }
     };
 
 }
