@@ -28,6 +28,8 @@ struct PassOptions {
     std::filesystem::path mod_files_dir;
     std::vector<std::filesystem::path> include_dirs;
 
+    int default_integer_kind = 4;
+
     std::string run_fun; // for global_stmts pass
     // TODO: Convert to std::filesystem::path (also change find_and_load_module())
     std::string runtime_library_dir;
@@ -48,6 +50,7 @@ struct PassOptions {
     bool intrinsic_symbols_mangling = false;
     bool all_symbols_mangling = false;
     bool bindc_mangling = false;
+    bool fortran_mangling = false;
     bool mangle_underscore = false;
     bool json = false;
     bool no_loc = false;
@@ -57,8 +60,6 @@ struct PassOptions {
 };
 
 struct CompilerOptions {
-    std::vector<std::string> runtime_linker_paths;
-
     // TODO: Convert to std::filesystem::path (also change find_and_load_module())
     PassOptions po;
 
@@ -67,10 +68,15 @@ struct CompilerOptions {
     bool c_preprocessor = false;
     std::vector<std::string> c_preprocessor_defines;
     bool prescan = true;
+    bool disable_main = false;
     bool symtab_only = false;
     bool show_stacktrace = false;
     bool use_colors = true;
     bool indent = true;
+    bool json = false;
+    bool tree = false;
+    bool visualize = false;
+    bool fast = false;
     bool openmp = false;
     bool generate_object_code = false;
     bool no_warnings = false;
@@ -83,6 +89,7 @@ struct CompilerOptions {
     bool implicit_argument_casting = false;
     bool print_leading_space = false;
     bool rtlib = false;
+    bool use_loop_variable_after_loop = false;
     std::string target = "";
     std::string arg_o = "";
     bool emit_debug_info = false;
